import warnings


warnings.warn(
    "The atlassian_jwt_auth.contrib.django package is deprecated in 4.0.0 "
    "in favour of atlassian_jwt_auth.frameworks.django.",
    DeprecationWarning, stacklevel=2
)
